# Copyright 2011-2018 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import (ResolverPlayground,
	ResolverPlaygroundTestCase)

class CompleteGraphTestCase(TestCase):

	def testCompleteGraphUseChange(self):
		"""
		Prevent reverse dependency breakage triggered by USE changes.
		"""

		ebuilds = {
			"dev-libs/libxml2-2.8.0": {
				"EAPI": "2",
				"IUSE": "+icu",
				"SLOT": "2",
			},
			"x11-libs/qt-webkit-4.8.2": {
				"EAPI": "2",
				"IUSE": "icu",
				"RDEPEND" : "dev-libs/libxml2:2[!icu?]",
			},
		}

		installed = {
			"dev-libs/libxml2-2.8.0": {
				"EAPI": "2",
				"IUSE": "+icu",
				"USE": "",
				"SLOT": "2",
			},
			"x11-libs/qt-webkit-4.8.2": {
				"EAPI": "2",
				"IUSE": "icu",
				"RDEPEND" : "dev-libs/libxml2:2[-icu]",
				"USE": "",
			}
		}

		world = ["x11-libs/qt-webkit"]

		test_cases = (

			ResolverPlaygroundTestCase(
				["dev-libs/libxml2"],
				options = {"--complete-graph-if-new-use" : "y" },
				mergelist = ["dev-libs/libxml2-2.8.0"],
				slot_collision_solutions = [{'dev-libs/libxml2-2.8.0': {'icu': False}}],
				success = False,
			),

			ResolverPlaygroundTestCase(
				["dev-libs/libxml2"],
				options = {"--complete-graph-if-new-use" : "n" },
				mergelist = ["dev-libs/libxml2-2.8.0"],
				success = True,
			),

			ResolverPlaygroundTestCase(
				["dev-libs/libxml2"],
				options = {"--ignore-world" : True},
				mergelist = ["dev-libs/libxml2-2.8.0"],
				success = True,
			),
		)

		playground = ResolverPlayground(ebuilds=ebuilds,
			installed=installed, world=world, debug=False)

		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.cleanup()

	def testCompleteGraphVersionChange(self):
		"""
		Prevent reverse dependency breakage triggered by version changes.
		"""

		ebuilds = {
			"sys-libs/x-0.1": {},
			"sys-libs/x-1": {},
			"sys-libs/x-2": {},
			"sys-apps/a-1": {"RDEPEND" : ">=sys-libs/x-1 <sys-libs/x-2"},
		}

		installed = {
			"sys-libs/x-1": {},
			"sys-apps/a-1": {"RDEPEND" : ">=sys-libs/x-1 <sys-libs/x-2"},
		}

		world = ["sys-apps/a"]

		test_cases = (
			ResolverPlaygroundTestCase(
				[">=sys-libs/x-2"],
				options = {"--complete-graph-if-new-ver" : "n", "--rebuild-if-new-slot": "n"},
				mergelist = ["sys-libs/x-2"],
				success = True,
			),
			ResolverPlaygroundTestCase(
				[">=sys-libs/x-2"],
				options = {"--ignore-world" : True},
				mergelist = ["sys-libs/x-2"],
				success = True,
			),
			ResolverPlaygroundTestCase(
				[">=sys-libs/x-2"],
				options = {"--complete-graph-if-new-ver" : "y"},
				mergelist = ["sys-libs/x-2"],
				slot_collision_solutions = [],
				success = False,
			),
			ResolverPlaygroundTestCase(
				["<sys-libs/x-1"],
				options = {"--complete-graph-if-new-ver" : "n", "--rebuild-if-new-slot": "n"},
				mergelist = ["sys-libs/x-0.1"],
				success = True,
			),
			ResolverPlaygroundTestCase(
				["<sys-libs/x-1"],
				options = {"--ignore-world" : True},
				mergelist = ["sys-libs/x-0.1"],
				success = True,
			),
			ResolverPlaygroundTestCase(
				["<sys-libs/x-1"],
				options = {"--complete-graph-if-new-ver" : "y"},
				mergelist = ["sys-libs/x-0.1"],
				slot_collision_solutions = [],
				success = False,
			),
		)

		playground = ResolverPlayground(ebuilds=ebuilds,
			installed=installed, world=world, debug=False)

		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.cleanup()



# Copyright 2017 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.dep import Atom, use_reduce
from portage.dep.dep_check import _overlap_dnf

class OverlapDNFTestCase(TestCase):

	def testOverlapDNF(self):

		test_cases = (
			(
				'|| ( cat/A cat/B ) cat/E || ( cat/C cat/D )',
				[['||', 'cat/A', 'cat/B'], 'cat/E', ['||', 'cat/C', 'cat/D']],
			),
			(
				'|| ( cat/A cat/B ) cat/D || ( cat/B cat/C )',
				['cat/D', ['||', ['cat/A', 'cat/B'], ['cat/A', 'cat/C'], ['cat/B', 'cat/B'], ['cat/B', 'cat/C']]],
			),
			(
				'|| ( cat/A cat/B ) || ( cat/C cat/D )  || ( ( cat/B cat/E ) cat/F )',
				[['||', ['cat/A', 'cat/B', 'cat/E'], ['cat/A', 'cat/F'], ['cat/B', 'cat/B', 'cat/E'], ['cat/B', 'cat/F']], ['||', 'cat/C', 'cat/D']],
			),
		)

		for dep_str, result in test_cases:
			self.assertEqual(_overlap_dnf(use_reduce(dep_str, token_class=Atom, opconvert=True)), result)



# Copyright 2018 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

import difflib
import os
import subprocess

import portage
from portage.const import PORTAGE_BIN_PATH
from portage.tests import TestCase


class TestFilterBashEnv(TestCase):
	def testTestFilterBashEnv(self):

		test_cases = (
			(
				'RDEPEND BASH.* _EPATCH_ECLASS',
				br'''declare -ir BASHPID="28997"
declare -rx A="portage-2.3.24.tar.bz2"
declare -- DESKTOP_DATABASE_DIR="/usr/share/applications"
declare PDEPEND="
        !build? (
                >=net-misc/rsync-2.6.4
                userland_GNU? ( >=sys-apps/coreutils-6.4 )
        ) "
declare RDEPEND="
        >=app-arch/tar-1.27
        dev-lang/python-exec:2"
declare -x PF="portage-2.3.24"
declare -a PYTHON_COMPAT=([0]="pypy" [1]="python3_4" [2]="python3_5" [3]="python3_6" [4]="python2_7")
declare -- _EPATCH_ECLASS="1"
declare -- _EUTILS_ECLASS="1"
declare -- f
get_libdir ()
{
    local CONF_LIBDIR;
    if [ -n "${CONF_LIBDIR_OVERRIDE}" ]; then
        echo ${CONF_LIBDIR_OVERRIDE};
    else
        get_abi_LIBDIR;
    fi
}
make_wrapper ()
{
    cat  <<-EOF
export ${var}="\${${var}}:${EPREFIX}${libdir}"
EOF
}
use_if_iuse ()
{
    in_iuse $1 || return 1;
    use $1
}
''',
				br'''declare -x A="portage-2.3.24.tar.bz2"
declare -- DESKTOP_DATABASE_DIR="/usr/share/applications"
declare PDEPEND="
        !build? (
                >=net-misc/rsync-2.6.4
                userland_GNU? ( >=sys-apps/coreutils-6.4 )
        ) "
declare -x PF="portage-2.3.24"
declare -a PYTHON_COMPAT=([0]="pypy" [1]="python3_4" [2]="python3_5" [3]="python3_6" [4]="python2_7")
declare -- _EUTILS_ECLASS="1"
declare -- f
get_libdir ()
{
    local CONF_LIBDIR;
    if [ -n "${CONF_LIBDIR_OVERRIDE}" ]; then
        echo ${CONF_LIBDIR_OVERRIDE};
    else
        get_abi_LIBDIR;
    fi
}
make_wrapper ()
{
    cat  <<-EOF
export ${var}="\${${var}}:${EPREFIX}${libdir}"
EOF
}
use_if_iuse ()
{
    in_iuse $1 || return 1;
    use $1
}
'''),
		)

		for filter_vars, env_in, env_out in test_cases:
			proc = None
			try:
				proc = subprocess.Popen(
					[
						portage._python_interpreter,
						os.path.join(PORTAGE_BIN_PATH, 'filter-bash-environment.py'),
						filter_vars,
					],
					stdin=subprocess.PIPE,
					stdout=subprocess.PIPE,
				)
				proc.stdin.write(env_in)
				proc.stdin.close()
				result = proc.stdout.read()
			finally:
				if proc is not None:
					proc.stdin.close()
					proc.wait()
					proc.stdout.close()

			diff = list(difflib.unified_diff(
				env_out.decode('utf_8').splitlines(),
				result.decode('utf_8').splitlines()))

			self.assertEqual(diff, [])
